@@ -142,6 +142,10 @@ class QtViewerDockWidget(QDockWidget):
         self.setTitleBarWidget(self.title)
         self.visibilityChanged.connect(self._on_visibility_changed)
 
+    def destroyOnClose(self):
+        """Destroys dock plugin dock widget when 'x' is clicked."""
+        self.qt_viewer.viewer.window.remove_dock_widget(self)
+
     def _maybe_add_vertical_stretch(self, widget):
         """Add vertical stretch to the bottom of a vertical layout only
 
@@ -217,6 +221,23 @@ class QtViewerDockWidget(QDockWidget):
         return self.size().height() > self.size().width()
 
     def _on_visibility_changed(self, visible):
+        try:
+            actions = [
+                action.text()
+                for action in self.qt_viewer.viewer.window.plugins_menu.actions()
+            ]
+            idx = actions.index(self.name)
+
+            current_action = (
+                self.qt_viewer.viewer.window.plugins_menu.actions()[idx]
+            )
+            current_action.setChecked(visible)
+            self.setVisible(visible)
+
+        except (AttributeError, ValueError):
+            # AttributeError: This error happens when the plugins menu is not yet built.
+            # ValueError: This error is when the action is from the windows menu.
+            pass
         if not visible:
             return
         with qt_signals_blocked(self):
@@ -258,13 +279,33 @@ class QtCustomTitleBar(QLabel):
         line = QFrame(self)
         line.setObjectName("QtCustomTitleBarLine")
 
-        self.close_button = QPushButton(self)
-        self.close_button.setToolTip(trans._('hide this panel'))
-        self.close_button.setObjectName("QTitleBarCloseButton")
-        self.close_button.setCursor(Qt.ArrowCursor)
-        self.close_button.clicked.connect(
-            lambda: self.parent().toggleViewAction().trigger()
-        )
+        add_close = False
+        try:
+            # if the plugins menu is already created, check to see if this is a plugin
+            # dock widget.  If it is, then add the close button option to the title bar.
+            actions = [
+                action.text()
+                for action in self.parent().qt_viewer.viewer.window.plugins_menu.actions()
+            ]
+            if self.parent().name in actions:
+                add_close = True
+                self.close_button = QPushButton(self)
+                self.close_button.setToolTip(trans._('close this panel'))
+                self.close_button.setObjectName("QTitleBarCloseButton")
+                self.close_button.setCursor(Qt.ArrowCursor)
+                self.close_button.clicked.connect(
+                    lambda: self.parent().destroyOnClose()
+                )
+            else:
+                add_close = False
+        except AttributeError:
+            pass
+        self.hide_button = QPushButton(self)
+        self.hide_button.setToolTip(trans._('hide this panel'))
+        self.hide_button.setObjectName("QTitleBarHideButton")
+        self.hide_button.setCursor(Qt.ArrowCursor)
+        self.hide_button.clicked.connect(lambda: self.parent().close())
+
         self.float_button = QPushButton(self)
         self.float_button.setToolTip(trans._('float this panel'))
         self.float_button.setObjectName("QTitleBarFloatButton")
@@ -282,7 +323,9 @@ class QtCustomTitleBar(QLabel):
             layout.setSpacing(4)
             layout.setContentsMargins(0, 8, 0, 8)
             line.setFixedWidth(1)
-            layout.addWidget(self.close_button, 0, Qt.AlignHCenter)
+            if add_close:
+                layout.addWidget(self.close_button, 0, Qt.AlignHCenter)
+            layout.addWidget(self.hide_button, 0, Qt.AlignHCenter)
             layout.addWidget(self.float_button, 0, Qt.AlignHCenter)
             layout.addWidget(line, 0, Qt.AlignHCenter)
             self.title.hide()
@@ -292,7 +335,10 @@ class QtCustomTitleBar(QLabel):
             layout.setSpacing(4)
             layout.setContentsMargins(8, 1, 8, 0)
             line.setFixedHeight(1)
-            layout.addWidget(self.close_button)
+            if add_close:
+                layout.addWidget(self.close_button)
+
+            layout.addWidget(self.hide_button)
             layout.addWidget(self.float_button)
             layout.addWidget(line)
             layout.addWidget(self.title)
