@@ -1,14 +1,15 @@
 import numpy as np
+import pytest
 
 
-def test_4D_5D_images(make_test_viewer):
+def test_4D_5D_images(make_napari_viewer):
     """Test adding 4D followed by 5D image layers to the viewer.
 
     Initially only 2 sliders should be present, then a third slider should be
     created.
     """
     np.random.seed(0)
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
     view = viewer.window.qt_viewer
 
     # add 4D image data
@@ -30,10 +31,10 @@ def test_4D_5D_images(make_test_viewer):
     assert np.sum(view.dims._displayed_sliders) == 3
 
 
-def test_5D_image_3D_rendering(make_test_viewer):
+def test_5D_image_3D_rendering(make_napari_viewer):
     """Test 3D rendering of a 5D image."""
     np.random.seed(0)
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
     view = viewer.window.qt_viewer
 
     # add 4D image data
@@ -54,12 +55,12 @@ def test_5D_image_3D_rendering(make_test_viewer):
     assert np.sum(view.dims._displayed_sliders) == 2
 
 
-def test_change_image_dims(make_test_viewer):
+def test_change_image_dims(make_napari_viewer):
     """Test changing the dims and shape of an image layer in place and checking
     the numbers of sliders and their ranges changes appropriately.
     """
     np.random.seed(0)
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
     view = viewer.window.qt_viewer
 
     # add 3D image data
@@ -96,14 +97,14 @@ def test_change_image_dims(make_test_viewer):
     assert np.sum(view.dims._displayed_sliders) == 1
 
 
-def test_range_one_image(make_test_viewer):
+def test_range_one_image(make_napari_viewer):
     """Test adding an image with a range one dimensions.
 
     There should be no slider shown for the axis corresponding to the range
     one dimension.
     """
     np.random.seed(0)
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
     view = viewer.window.qt_viewer
 
     # add 5D image data with range one dimensions
@@ -126,14 +127,14 @@ def test_range_one_image(make_test_viewer):
     assert np.sum(view.dims._displayed_sliders) == 3
 
 
-def test_range_one_images_and_points(make_test_viewer):
+def test_range_one_images_and_points(make_napari_viewer):
     """Test adding images with range one dimensions and points.
 
     Initially no sliders should be present as the images have range one
     dimensions. On adding the points the sliders should be displayed.
     """
     np.random.seed(0)
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
     view = viewer.window.qt_viewer
 
     # add 5D image data with range one dimensions
@@ -156,9 +157,10 @@ def test_range_one_images_and_points(make_test_viewer):
     assert np.sum(view.dims._displayed_sliders) == 3
 
 
-def test_update_console(make_test_viewer):
+@pytest.mark.filterwarnings("ignore::DeprecationWarning:jupyter_client")
+def test_update_console(make_napari_viewer):
     """Test updating the console with local variables."""
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
     view = viewer.window.qt_viewer
 
     # Check viewer in console
@@ -175,13 +177,13 @@ def test_update_console(make_test_viewer):
     assert view.console.shell.user_ns['b'] == b
 
 
-def test_changing_display_surface(make_test_viewer):
+def test_changing_display_surface(make_napari_viewer):
     """Test adding 3D surface and changing its display."""
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
     view = viewer.window.qt_viewer
 
     np.random.seed(0)
-    vertices = np.random.random((10, 3))
+    vertices = 20 * np.random.random((10, 3))
     faces = np.random.randint(10, size=(6, 3))
     values = np.random.random(10)
     data = (vertices, faces, values)
@@ -191,7 +193,7 @@ def test_changing_display_surface(make_test_viewer):
     )
 
     assert len(viewer.layers) == 1
-    assert view.layers.vbox_layout.count() == 2 * len(viewer.layers) + 2
+    assert view.layers.model().rowCount() == len(viewer.layers)
 
     assert viewer.dims.ndim == 3
     assert view.dims.nsliders == viewer.dims.ndim
@@ -216,9 +218,9 @@ def test_changing_display_surface(make_test_viewer):
         viewer.dims.set_point(0, s)
 
 
-def test_labels_undo_redo(make_test_viewer):
+def test_labels_undo_redo(make_napari_viewer):
     """Test undoing/redoing on the labels layer."""
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
 
     data = np.zeros((50, 50), dtype=np.uint8)
     data[:5, :5] = 1
@@ -245,6 +247,7 @@ def test_labels_undo_redo(make_test_viewer):
 
     # history limit
     labels._history_limit = 1
+    labels._reset_history()
     labels.fill((0, 0), 3)
 
     l3 = labels.data.copy()
@@ -257,3 +260,19 @@ def test_labels_undo_redo(make_test_viewer):
     # cannot undo as limit exceeded
     labels.undo()
     assert np.array_equal(l2, labels.data)
+
+
+def test_labels_brush_size(make_napari_viewer):
+    """Test changing labels brush size."""
+    viewer = make_napari_viewer()
+
+    data = np.zeros((50, 50), dtype=np.uint8)
+    labels = viewer.add_labels(data)
+
+    # Make small change
+    labels.brush_size = 20
+    assert labels.brush_size == 20
+
+    # Make large change
+    labels.brush_size = 100
+    assert labels.brush_size == 100
