@@ -1,9 +1,11 @@
 import numpy as np
 
+from napari._tests.utils import skip_on_win_ci, slow
 
-def test_image_rendering(make_test_viewer):
+
+def test_image_rendering(make_napari_viewer):
     """Test 3D image with different rendering."""
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
 
     viewer.dims.ndisplay = 3
 
@@ -35,3 +37,23 @@ def test_image_rendering(make_test_viewer):
     # Change rendering property
     layer.rendering = 'additive'
     assert layer.rendering == 'additive'
+
+
+@slow(15)
+@skip_on_win_ci
+def test_visibility_consistency(qtbot, make_napari_viewer):
+    """Make sure toggling visibility maintains image contrast.
+
+    see #1622 for details.
+    """
+    viewer = make_napari_viewer(show=True)
+
+    layer = viewer.add_image(
+        np.random.random((200, 200)), contrast_limits=[0, 10]
+    )
+    qtbot.wait(10)
+    layer.contrast_limits = (0, 2)
+    screen1 = viewer.screenshot(flash=False).astype('float')
+    layer.visible = True
+    screen2 = viewer.screenshot(flash=False).astype('float')
+    assert np.max(np.abs(screen2 - screen1)) < 5
