@@ -1,12 +1,11 @@
 import numpy as np
-import os
-import sys
-import pytest
+
+from napari._tests.utils import skip_local_popups, skip_on_win_ci
 
 
-def test_multiscale(make_test_viewer):
+def test_multiscale(make_napari_viewer):
     """Test rendering of multiscale data."""
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
 
     shapes = [(4000, 3000), (2000, 1500), (1000, 750), (500, 375)]
     np.random.seed(0)
@@ -16,7 +15,7 @@ def test_multiscale(make_test_viewer):
 
     # Set canvas size to target amount
     viewer.window.qt_viewer.view.canvas.size = (800, 600)
-    list(viewer.window.qt_viewer.layer_to_visual.values())[0].on_draw(None)
+    viewer.window.qt_viewer.on_draw(None)
 
     # Check that current level is first large enough to fill the canvas with
     # a greater than one pixel depth
@@ -27,24 +26,24 @@ def test_multiscale(make_test_viewer):
     assert np.all(layer.corner_pixels[1] >= np.subtract(shapes[2], 1))
 
     # Test value at top left corner of image
-    layer.position = (0, 0)
-    value = layer.get_value()
+    viewer.cursor.position = (0, 0)
+    value = layer.get_value(viewer.cursor.position, world=True)
     np.testing.assert_allclose(value, (2, data[2][(0, 0)]))
 
     # Test value at bottom right corner of image
-    layer.position = (999, 749)
-    value = layer.get_value()
+    viewer.cursor.position = (3995, 2995)
+    value = layer.get_value(viewer.cursor.position, world=True)
     np.testing.assert_allclose(value, (2, data[2][(999, 749)]))
 
     # Test value outside image
-    layer.position = (1000, 750)
-    value = layer.get_value()
+    viewer.cursor.position = (4000, 3000)
+    value = layer.get_value(viewer.cursor.position, world=True)
     assert value[1] is None
 
 
-def test_3D_multiscale_image(make_test_viewer):
+def test_3D_multiscale_image(make_napari_viewer):
     """Test rendering of 3D multiscale image uses lowest resolution."""
-    viewer = make_test_viewer()
+    viewer = make_napari_viewer()
 
     data = [np.random.random((128,) * 3), np.random.random((64,) * 3)]
     viewer.add_image(data)
@@ -56,16 +55,14 @@ def test_3D_multiscale_image(make_test_viewer):
     assert viewer.layers[0].data_level == 1
 
     # Note that draw command must be explicitly triggered in our tests
-    list(viewer.window.qt_viewer.layer_to_visual.values())[0].on_draw(None)
+    viewer.window.qt_viewer.on_draw(None)
 
 
-@pytest.mark.skipif(
-    sys.platform.startswith('win') or not os.getenv("CI"),
-    reason='Screenshot tests are not supported on napari windows CI.',
-)
-def test_multiscale_screenshot(make_test_viewer):
+@skip_on_win_ci
+@skip_local_popups
+def test_multiscale_screenshot(make_napari_viewer):
     """Test rendering of multiscale data with screenshot."""
-    viewer = make_test_viewer(show=True)
+    viewer = make_napari_viewer(show=True)
 
     shapes = [(4000, 3000), (2000, 1500), (1000, 750), (500, 375)]
     data = [np.ones(s) for s in shapes]
@@ -74,8 +71,8 @@ def test_multiscale_screenshot(make_test_viewer):
     # Set canvas size to target amount
     viewer.window.qt_viewer.view.canvas.size = (800, 600)
 
-    screenshot = viewer.screenshot(canvas_only=True)
-    center_coord = np.round(np.array(screenshot.shape[:2]) / 2).astype(np.int)
+    screenshot = viewer.screenshot(canvas_only=True, flash=False)
+    center_coord = np.round(np.array(screenshot.shape[:2]) / 2).astype(int)
     target_center = np.array([255, 255, 255, 255], dtype='uint8')
     target_edge = np.array([0, 0, 0, 255], dtype='uint8')
     screen_offset = 3  # Offset is needed as our screenshots have black borders
@@ -89,13 +86,11 @@ def test_multiscale_screenshot(make_test_viewer):
     )
 
 
-@pytest.mark.skipif(
-    sys.platform.startswith('win') or not os.getenv("CI"),
-    reason='Screenshot tests are not supported on napari windows CI.',
-)
-def test_multiscale_screenshot_zoomed(make_test_viewer):
+@skip_on_win_ci
+@skip_local_popups
+def test_multiscale_screenshot_zoomed(make_napari_viewer):
     """Test rendering of multiscale data with screenshot after zoom."""
-    viewer = make_test_viewer(show=True)
+    viewer = make_napari_viewer(show=True)
     view = viewer.window.qt_viewer
 
     shapes = [(4000, 3000), (2000, 1500), (1000, 750), (500, 375)]
@@ -107,13 +102,13 @@ def test_multiscale_screenshot_zoomed(make_test_viewer):
 
     # Set zoom of camera to show highest resolution tile
     view.view.camera.rect = [1000, 1000, 200, 150]
-    list(view.layer_to_visual.values())[0].on_draw(None)
+    viewer.window.qt_viewer.on_draw(None)
 
     # Check that current level is bottom level of multiscale
     assert viewer.layers[0].data_level == 0
 
-    screenshot = viewer.screenshot(canvas_only=True)
-    center_coord = np.round(np.array(screenshot.shape[:2]) / 2).astype(np.int)
+    screenshot = viewer.screenshot(canvas_only=True, flash=False)
+    center_coord = np.round(np.array(screenshot.shape[:2]) / 2).astype(int)
     target_center = np.array([255, 255, 255, 255], dtype='uint8')
     screen_offset = 3  # Offset is needed as our screenshots have black borders
 
@@ -126,13 +121,11 @@ def test_multiscale_screenshot_zoomed(make_test_viewer):
     )
 
 
-@pytest.mark.skipif(
-    sys.platform.startswith('win') or not os.getenv("CI"),
-    reason='Screenshot tests are not supported on napari windows CI.',
-)
-def test_image_screenshot_zoomed(make_test_viewer):
+@skip_on_win_ci
+@skip_local_popups
+def test_image_screenshot_zoomed(make_napari_viewer):
     """Test rendering of image data with screenshot after zoom."""
-    viewer = make_test_viewer(show=True)
+    viewer = make_napari_viewer(show=True)
     view = viewer.window.qt_viewer
 
     data = np.ones((4000, 3000))
@@ -143,10 +136,10 @@ def test_image_screenshot_zoomed(make_test_viewer):
 
     # Set zoom of camera to show highest resolution tile
     view.view.camera.rect = [1000, 1000, 200, 150]
-    list(view.layer_to_visual.values())[0].on_draw(None)
+    viewer.window.qt_viewer.on_draw(None)
 
-    screenshot = viewer.screenshot(canvas_only=True)
-    center_coord = np.round(np.array(screenshot.shape[:2]) / 2).astype(np.int)
+    screenshot = viewer.screenshot(canvas_only=True, flash=False)
+    center_coord = np.round(np.array(screenshot.shape[:2]) / 2).astype(int)
     target_center = np.array([255, 255, 255, 255], dtype='uint8')
     screen_offset = 3  # Offset is needed as our screenshots have black borders
 
@@ -157,3 +150,55 @@ def test_image_screenshot_zoomed(make_test_viewer):
     np.testing.assert_allclose(
         screenshot[-screen_offset, -screen_offset], target_center
     )
+
+
+@skip_on_win_ci
+@skip_local_popups
+def test_5D_multiscale(make_napari_viewer):
+    """Test 5D multiscale data."""
+    # Show must be true to trigger multiscale draw and corner estimation
+    viewer = make_napari_viewer(show=True)
+    view = viewer.window.qt_viewer
+    view.set_welcome_visible(False)
+    shapes = [(1, 2, 5, 20, 20), (1, 2, 5, 10, 10), (1, 2, 5, 5, 5)]
+    np.random.seed(0)
+    data = [np.random.random(s) for s in shapes]
+    layer = viewer.add_image(data, multiscale=True)
+    assert layer.data == data
+    assert layer.multiscale is True
+    assert layer.ndim == len(shapes[0])
+
+
+@skip_on_win_ci
+@skip_local_popups
+def test_multiscale_flipped_axes(make_napari_viewer):
+    """Check rendering of multiscale images with negative scale values.
+
+    See https://github.com/napari/napari/issues/3057
+    """
+    viewer = make_napari_viewer(show=True)
+
+    shapes = [(4000, 3000), (2000, 1500), (1000, 750), (500, 375)]
+    data = [np.ones(s) for s in shapes]
+    # this used to crash, see issue #3057
+    _ = viewer.add_image(
+        data, multiscale=True, contrast_limits=[0, 1], scale=(-1, 1)
+    )
+
+
+@skip_on_win_ci
+@skip_local_popups
+def test_multiscale_rotated_image(make_napari_viewer):
+    viewer = make_napari_viewer(show=True)
+    sizes = [4000 // i for i in range(1, 5)]
+    arrays = [np.zeros((size, size), dtype=np.uint8) for size in sizes]
+    for arr in arrays:
+        arr[:10, :10] = 255
+        arr[-10:, -10:] = 255
+
+    viewer.add_image(arrays, multiscale=True, rotate=44)
+    screenshot_rgba = viewer.screenshot(canvas_only=True, flash=False)
+    screenshot_rgb = screenshot_rgba[..., :3]
+    assert np.any(
+        screenshot_rgb
+    )  # make sure there is at least one white pixel
