@@ -430,8 +430,8 @@ def test_world_extent():
     layers = LayerList()
 
     # Empty data is taken to be 512 x 512
-    np.testing.assert_allclose(layers.extent.world[0], (0, 0))
-    np.testing.assert_allclose(layers.extent.world[1], (511, 511))
+    np.testing.assert_allclose(layers.extent.world[0], (-0.5, -0.5))
+    np.testing.assert_allclose(layers.extent.world[1], (511.5, 511.5))
     np.testing.assert_allclose(layers.extent.step, (1, 1))
 
     # Add one layer
@@ -439,10 +439,10 @@ def test_world_extent():
         np.random.random((6, 10, 15)), scale=(3, 1, 1), translate=(10, 20, 5)
     )
     layers.append(layer_a)
-    np.testing.assert_allclose(layer_a.extent.world[0], (10, 20, 5))
-    np.testing.assert_allclose(layer_a.extent.world[1], (25, 29, 19))
-    np.testing.assert_allclose(layers.extent.world[0], (10, 20, 5))
-    np.testing.assert_allclose(layers.extent.world[1], (25, 29, 19))
+    np.testing.assert_allclose(layer_a.extent.world[0], (8.5, 19.5, 4.5))
+    np.testing.assert_allclose(layer_a.extent.world[1], (26.5, 29.5, 19.5))
+    np.testing.assert_allclose(layers.extent.world[0], (8.5, 19.5, 4.5))
+    np.testing.assert_allclose(layers.extent.world[1], (26.5, 29.5, 19.5))
     np.testing.assert_allclose(layers.extent.step, (3, 1, 1))
 
     # Add another layer
@@ -450,10 +450,10 @@ def test_world_extent():
         np.random.random((8, 6, 15)), scale=(6, 2, 1), translate=(-5, -10, 10)
     )
     layers.append(layer_b)
-    np.testing.assert_allclose(layer_b.extent.world[0], (-5, -10, 10))
-    np.testing.assert_allclose(layer_b.extent.world[1], (37, 0, 24))
-    np.testing.assert_allclose(layers.extent.world[0], (-5, -10, 5))
-    np.testing.assert_allclose(layers.extent.world[1], (37, 29, 24))
+    np.testing.assert_allclose(layer_b.extent.world[0], (-8, -11, 9.5))
+    np.testing.assert_allclose(layer_b.extent.world[1], (40, 1, 24.5))
+    np.testing.assert_allclose(layers.extent.world[0], (-8, -11, 4.5))
+    np.testing.assert_allclose(layers.extent.world[1], (40, 29.5, 24.5))
     np.testing.assert_allclose(layers.extent.step, (3, 1, 1))
 
 
@@ -465,12 +465,18 @@ def test_world_extent_mixed_ndim():
     # Add 3D layer
     layer_a = Image(np.random.random((15, 15, 15)), scale=(4, 12, 2))
     layers.append(layer_a)
-    np.testing.assert_allclose(layers.extent.world[1], (56, 168, 28))
+    np.testing.assert_allclose(layers.extent.world[1], (58, 174, 29))
+    np.testing.assert_allclose(
+        layers.extent.world[1] - layers.extent.world[0], (60, 180, 30)
+    )
 
     # Add 2D layer
     layer_b = Image(np.random.random((10, 10)), scale=(6, 4))
     layers.append(layer_b)
-    np.testing.assert_allclose(layers.extent.world[1], (56, 168, 36))
+    np.testing.assert_allclose(layers.extent.world[1], (58, 174, 38))
+    np.testing.assert_allclose(
+        layers.extent.world[1] - layers.extent.world[0], (60, 180, 40)
+    )
     np.testing.assert_allclose(layers.extent.step, (4, 6, 2))
 
 
@@ -518,26 +524,3 @@ def test_name_uniqueness():
     layers.append(Image(np.random.random((10, 15)), name="Image"))
     layers.append(Image(np.random.random((10, 15)), name="Image"))
     assert [x.name for x in layers] == ['Image [1]', 'Image', 'Image [2]']
-
-
-def test_deprecated_selection():
-    """Test deprecated layer selection emits warnings."""
-    layers = LayerList()
-    layer = Image(np.random.random((10, 15)))
-    events = []
-
-    with pytest.warns(FutureWarning):
-        layer.events.select.connect(lambda e: events.append(e))
-        layer.events.deselect.connect(lambda e: events.append(e))
-
-    layers.append(layer)
-    assert events[-1].type == 'select'
-
-    with pytest.warns(FutureWarning):
-        assert layer.selected is True
-
-    with pytest.warns(FutureWarning):
-        layer.selected = False
-
-    assert events[-1].type == 'deselect'
-    assert layer not in layers.selection
