@@ -1,4 +1,6 @@
 from enum import auto
+from typing import Tuple
+
 from ...utils.misc import StringEnum
 
 
@@ -8,7 +10,7 @@ class Interpolation(StringEnum):
     The spatial filters used for interpolation are from vispy's
     spatial filters. The filters are built in the file below:
 
-    https://github.com/vispy/vispy/blob/master/vispy/glsl/build-spatial-filters.py
+    https://github.com/vispy/vispy/blob/main/vispy/glsl/build-spatial-filters.py
     """
 
     BESSEL = auto()
@@ -27,6 +29,16 @@ class Interpolation(StringEnum):
     SPLINE16 = auto()
     SPLINE36 = auto()
 
+    @classmethod
+    def view_subset(cls):
+        return (
+            cls.BICUBIC,
+            cls.BILINEAR,
+            cls.KAISER,
+            cls.NEAREST,
+            cls.SPLINE36,
+        )
+
 
 class Interpolation3D(StringEnum):
     """INTERPOLATION: Vispy interpolation mode for volume rendering."""
@@ -43,6 +55,8 @@ class Rendering(StringEnum):
               the result is opaque.
             * mip: maximum intensity projection. Cast a ray and display the
               maximum value that was encountered.
+            * minip: minimum intensity projection. Cast a ray and display the
+              minimum value that was encountered.
             * attenuated_mip: attenuated maximum intensity projection. Cast a
               ray and attenuate values based on integral of encountered values,
               display the maximum value that was encountered after attenuation.
@@ -52,10 +66,35 @@ class Rendering(StringEnum):
             * iso: isosurface. Cast a ray until a certain threshold is
               encountered. At that location, lighning calculations are
               performed to give the visual appearance of a surface.
+            * iso_categorical: isosurface for categorical data (e.g., labels).
+              Cast a ray until a non-background value is encountered. At that
+              location, lighning calculations are performed to give the visual
+              appearance of a surface.
+            * average: average intensity projection. Cast a ray and display the
+              average of values that were encountered.
     """
 
     TRANSLUCENT = auto()
     ADDITIVE = auto()
     ISO = auto()
+    ISO_CATEGORICAL = auto()
     MIP = auto()
+    MINIP = auto()
     ATTENUATED_MIP = auto()
+    AVERAGE = auto()
+
+    @classmethod
+    def image_layer_subset(cls) -> Tuple['Rendering']:
+        return (
+            cls.TRANSLUCENT,
+            cls.ADDITIVE,
+            cls.ISO,
+            cls.MIP,
+            cls.MINIP,
+            cls.ATTENUATED_MIP,
+            cls.AVERAGE,
+        )
+
+    @classmethod
+    def labels_layer_subset(cls) -> Tuple['Rendering']:
+        return (cls.TRANSLUCENT, cls.ISO_CATEGORICAL)
