@@ -1,10 +1,14 @@
-import numpy as np
 import dask.array as da
+import numpy as np
+import pytest
+import tensorstore as ts
 import xarray as xr
 
-import pytest
-from vispy.color import Colormap
+from napari._tests.utils import check_layer_world_data_extent
 from napari.layers import Image
+from napari.layers.utils.plane import ClippingPlaneList, SlicingPlane
+from napari.utils import Colormap
+from napari.utils.transforms.transform_utils import rotate_to_matrix
 
 
 def test_random_image():
@@ -15,8 +19,7 @@ def test_random_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
-    assert layer.dims.range == [(0, m, 1) for m in shape]
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer.multiscale is False
     assert layer._data_view.shape == shape[-2:]
@@ -31,8 +34,7 @@ def test_negative_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
-    assert layer.dims.range == [(0, m, 1) for m in shape]
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.shape == shape[-2:]
 
@@ -41,8 +43,7 @@ def test_negative_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
-    assert layer.dims.range == [(0, m, 1) for m in shape]
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.shape == shape[-2:]
 
@@ -54,7 +55,7 @@ def test_all_zeros_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.shape == shape[-2:]
 
@@ -67,7 +68,7 @@ def test_integer_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.shape == shape[-2:]
 
@@ -79,7 +80,7 @@ def test_bool_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.shape == shape[-2:]
 
@@ -92,7 +93,7 @@ def test_3D_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.shape == shape[-2:]
 
@@ -105,7 +106,7 @@ def test_3D_image_shape_1():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.shape == shape[-2:]
 
@@ -118,7 +119,7 @@ def test_4D_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.shape == shape[-2:]
 
@@ -131,7 +132,7 @@ def test_5D_image_shape_1():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.shape == shape[-2:]
 
@@ -144,7 +145,7 @@ def test_rgb_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape) - 1
-    assert layer.shape == shape[:-1]
+    np.testing.assert_array_equal(layer.extent.data[1], shape[:-1])
     assert layer.rgb is True
     assert layer._data_view.shape == shape[-3:]
 
@@ -157,7 +158,7 @@ def test_rgba_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape) - 1
-    assert layer.shape == shape[:-1]
+    np.testing.assert_array_equal(layer.extent.data[1], shape[:-1])
     assert layer.rgb is True
     assert layer._data_view.shape == shape[-3:]
 
@@ -171,7 +172,7 @@ def test_negative_rgba_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape) - 1
-    assert layer.shape == shape[:-1]
+    np.testing.assert_array_equal(layer.extent.data[1], shape[:-1])
     assert layer.rgb is True
     assert layer._data_view.shape == shape[-3:]
 
@@ -180,7 +181,7 @@ def test_negative_rgba_image():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape) - 1
-    assert layer.shape == shape[:-1]
+    np.testing.assert_array_equal(layer.extent.data[1], shape[:-1])
     assert layer.rgb is True
     assert layer._data_view.shape == shape[-3:]
 
@@ -193,7 +194,7 @@ def test_non_rgb_image():
     layer = Image(data, rgb=False)
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.shape == shape[-2:]
 
@@ -220,8 +221,7 @@ def test_changing_image():
     layer.data = data_b
     assert np.all(layer.data == data_b)
     assert layer.ndim == len(shape_b)
-    assert layer.shape == shape_b
-    assert layer.dims.range == [(0, m, 1) for m in shape_b]
+    np.testing.assert_array_equal(layer.extent.data[1], shape_b)
     assert layer.rgb is False
     assert layer._data_view.shape == shape_b[-2:]
 
@@ -239,8 +239,7 @@ def test_changing_image_dims():
     layer.data = data_b
     assert np.all(layer.data == data_b)
     assert layer.ndim == len(shape_b)
-    assert layer.shape == shape_b
-    assert layer.dims.range == [(0, m, 1) for m in shape_b]
+    np.testing.assert_array_equal(layer.extent.data[1], shape_b)
     assert layer.rgb is False
     assert layer._data_view.shape == shape_b[-2:]
 
@@ -329,36 +328,36 @@ def test_colormaps():
     np.random.seed(0)
     data = np.random.random((10, 15))
     layer = Image(data)
-    assert layer.colormap[0] == 'gray'
-    assert type(layer.colormap[1]) == Colormap
+    assert layer.colormap.name == 'gray'
+    assert isinstance(layer.colormap, Colormap)
 
     layer.colormap = 'magma'
-    assert layer.colormap[0] == 'magma'
-    assert type(layer.colormap[1]) == Colormap
+    assert layer.colormap.name == 'magma'
+    assert isinstance(layer.colormap, Colormap)
 
     cmap = Colormap([[0.0, 0.0, 0.0, 0.0], [0.3, 0.7, 0.2, 1.0]])
     layer.colormap = 'custom', cmap
-    assert layer.colormap[0] == 'custom'
-    assert layer.colormap[1] == cmap
+    assert layer.colormap.name == 'custom'
+    assert layer.colormap == cmap
 
     cmap = Colormap([[0.0, 0.0, 0.0, 0.0], [0.7, 0.2, 0.6, 1.0]])
     layer.colormap = {'new': cmap}
-    assert layer.colormap[0] == 'new'
-    assert layer.colormap[1] == cmap
+    assert layer.colormap.name == 'new'
+    assert layer.colormap == cmap
 
     layer = Image(data, colormap='magma')
-    assert layer.colormap[0] == 'magma'
-    assert type(layer.colormap[1]) == Colormap
+    assert layer.colormap.name == 'magma'
+    assert isinstance(layer.colormap, Colormap)
 
     cmap = Colormap([[0.0, 0.0, 0.0, 0.0], [0.3, 0.7, 0.2, 1.0]])
     layer = Image(data, colormap=('custom', cmap))
-    assert layer.colormap[0] == 'custom'
-    assert layer.colormap[1] == cmap
+    assert layer.colormap.name == 'custom'
+    assert layer.colormap == cmap
 
     cmap = Colormap([[0.0, 0.0, 0.0, 0.0], [0.7, 0.2, 0.6, 1.0]])
     layer = Image(data, colormap={'new': cmap})
-    assert layer.colormap[0] == 'new'
-    assert layer.colormap[1] == cmap
+    assert layer.colormap.name == 'new'
+    assert layer.colormap == cmap
 
 
 def test_contrast_limits():
@@ -514,17 +513,50 @@ def test_value():
     np.random.seed(0)
     data = np.random.random((10, 15))
     layer = Image(data)
-    value = layer.get_value()
-    assert layer.coordinates == (0, 0)
+    value = layer.get_value((0,) * 2)
     assert value == data[0, 0]
 
 
+@pytest.mark.parametrize(
+    'position,view_direction,dims_displayed,world',
+    [
+        ((0, 0, 0), [1, 0, 0], [0, 1, 2], False),
+        ((0, 0, 0), [1, 0, 0], [0, 1, 2], True),
+        ((0, 0, 0, 0), [0, 1, 0, 0], [1, 2, 3], True),
+    ],
+)
+def test_value_3d(position, view_direction, dims_displayed, world):
+    """Currently get_value should return None in 3D"""
+    np.random.seed(0)
+    data = np.random.random((10, 15, 15))
+    layer = Image(data)
+    layer._slice_dims([0, 0, 0], ndisplay=3)
+    value = layer.get_value(
+        position,
+        view_direction=view_direction,
+        dims_displayed=dims_displayed,
+        world=world,
+    )
+    assert value is None
+
+
 def test_message():
     """Test converting value and coords to message."""
     np.random.seed(0)
     data = np.random.random((10, 15))
     layer = Image(data)
-    msg = layer.get_message()
+    msg = layer.get_status((0,) * 2)
+    assert type(msg) == str
+
+
+def test_message_3d():
+    """Test converting values and coords to message in 3D."""
+    np.random.seed(0)
+    data = np.random.random((10, 15, 15))
+    layer = Image(data)
+    msg = layer.get_status(
+        (0, 0, 0), view_direction=[1, 0, 0], dims_displayed=[0, 1, 2]
+    )
     assert type(msg) == str
 
 
@@ -603,6 +635,20 @@ def test_image_translate(translate):
     Image(data, translate=translate)
 
 
+def test_image_scale_broadcast():
+    """Test scale is broadcast."""
+    data = np.random.random((5, 10, 15))
+    layer = Image(data, scale=(2, 2))
+    np.testing.assert_almost_equal(layer.scale, (1, 2, 2))
+
+
+def test_image_translate_broadcast():
+    """Test translate is broadcast."""
+    data = np.random.random((5, 10, 15))
+    layer = Image(data, translate=(2, 2))
+    np.testing.assert_almost_equal(layer.translate, (0, 2, 2))
+
+
 def test_grid_translate():
     np.random.seed(0)
     data = np.random.random((10, 15))
@@ -610,3 +656,139 @@ def test_grid_translate():
     translate = np.array([15, 15])
     layer.translate_grid = translate
     np.testing.assert_allclose(layer.translate_grid, translate)
+
+
+def test_world_data_extent():
+    """Test extent after applying transforms."""
+    np.random.seed(0)
+    shape = (6, 10, 15)
+    data = np.random.random(shape)
+    layer = Image(data)
+    extent = np.array(((0,) * 3, shape))
+    check_layer_world_data_extent(layer, extent, (3, 1, 1), (10, 20, 5), True)
+
+
+def test_data_to_world_2d_scale_translate_affine_composed():
+    data = np.ones((4, 3))
+    scale = (3, 2)
+    translate = (-4, 8)
+    affine = [[4, 0, 0], [0, 1.5, 0], [0, 0, 1]]
+
+    image = Image(data, scale=scale, translate=translate, affine=affine)
+
+    np.testing.assert_array_equal(image.scale, scale)
+    np.testing.assert_array_equal(image.translate, translate)
+    np.testing.assert_array_equal(image.affine, affine)
+    np.testing.assert_almost_equal(
+        image._data_to_world.affine_matrix,
+        ((12, 0, -16), (0, 3, 12), (0, 0, 1)),
+    )
+
+
+@pytest.mark.parametrize('scale', ((1, 1), (-1, 1), (1, -1), (-1, -1)))
+@pytest.mark.parametrize('angle_degrees', range(-180, 180, 30))
+def test_rotate_with_reflections_in_scale(scale, angle_degrees):
+    # See the GitHub issue for more details:
+    # https://github.com/napari/napari/issues/2984
+    data = np.ones((4, 3))
+    rotate = rotate_to_matrix(angle_degrees, ndim=2)
+
+    image = Image(data, scale=scale, rotate=rotate)
+
+    np.testing.assert_array_equal(image.scale, scale)
+    np.testing.assert_array_equal(image.rotate, rotate)
+
+
+def test_2d_image_with_channels_and_2d_scale_translate_then_scale_translate_padded():
+    # See the GitHub issue for more details:
+    # https://github.com/napari/napari/issues/2973
+    image = Image(np.ones((20, 20, 2)), scale=(1, 1), translate=(3, 4))
+
+    np.testing.assert_array_equal(image.scale, (1, 1, 1))
+    np.testing.assert_array_equal(image.translate, (0, 3, 4))
+
+
+@pytest.mark.parametrize('affine_size', range(3, 6))
+def test_2d_image_with_channels_and_affine_broadcasts(affine_size):
+    # For more details, see the GitHub issue:
+    # https://github.com/napari/napari/issues/3045
+    image = Image(np.ones((1, 1, 1, 100, 100)), affine=np.eye(affine_size))
+    np.testing.assert_array_equal(image.affine, np.eye(6))
+
+
+@pytest.mark.parametrize('affine_size', range(3, 6))
+def test_2d_image_with_channels_and_affine_assignment_broadcasts(affine_size):
+    # For more details, see the GitHub issue:
+    # https://github.com/napari/napari/issues/3045
+    image = Image(np.ones((1, 1, 1, 100, 100)))
+    image.affine = np.eye(affine_size)
+    np.testing.assert_array_equal(image.affine, np.eye(6))
+
+
+def test_image_state_update():
+    """Test that an image can be updated from the output of its
+    _get_state method()
+    """
+    image = Image(np.ones((32, 32, 32)))
+    state = image._get_state()
+    for k, v in state.items():
+        setattr(image, k, v)
+
+
+def test_instiantiate_with_experimental_slicing_plane_dict():
+    """Test that an image layer can be instantiated with plane parameters
+    in a dictionary.
+    """
+    plane_parameters = {
+        'position': (32, 32, 32),
+        'normal': (1, 1, 1),
+        'thickness': 22,
+    }
+    image = Image(
+        np.ones((32, 32, 32)), experimental_slicing_plane=plane_parameters
+    )
+    for k, v in plane_parameters.items():
+        if k == 'normal':
+            v = tuple(v / np.linalg.norm(v))
+        assert v == getattr(image.experimental_slicing_plane, k, v)
+
+
+def test_instiantiate_with_experimental_slicing_plane():
+    """Test that an image layer can be instantiated with plane parameters
+    in a Plane.
+    """
+    plane = SlicingPlane(position=(32, 32, 32), normal=(1, 1, 1), thickness=22)
+    image = Image(np.ones((32, 32, 32)), experimental_slicing_plane=plane)
+    for k, v in plane.dict().items():
+        assert v == getattr(image.experimental_slicing_plane, k, v)
+
+
+def test_instantiate_with_clipping_planelist():
+    planes = ClippingPlaneList.from_array(np.ones((2, 2, 3)))
+    image = Image(np.ones((32, 32, 32)), experimental_clipping_planes=planes)
+    assert len(image.experimental_clipping_planes) == 2
+
+
+def test_instantiate_with_experimental_clipping_planes_dict():
+    planes = [
+        {'position': (0, 0, 0), 'normal': (0, 0, 1)},
+        {'position': (0, 1, 0), 'normal': (1, 0, 0)},
+    ]
+    image = Image(np.ones((32, 32, 32)), experimental_clipping_planes=planes)
+    for i in range(len(planes)):
+        assert (
+            image.experimental_clipping_planes[i].position
+            == planes[i]['position']
+        )
+        assert (
+            image.experimental_clipping_planes[i].normal == planes[i]['normal']
+        )
+
+
+def test_tensorstore_image():
+    """Test an image coming from a tensorstore array."""
+    data = ts.array(
+        np.full(shape=(1024, 1024), fill_value=255, dtype=np.uint8)
+    )
+    layer = Image(data)
+    assert np.all(layer.data == data)
