@@ -1,6 +1,8 @@
 import dask.array as da
 import numpy as np
 import pytest
+from hypothesis import given
+from hypothesis.extra.numpy import array_shapes
 from skimage.transform import pyramid_gaussian
 
 from napari.layers.image._image_utils import guess_multiscale, guess_rgb
@@ -24,6 +26,11 @@ def test_guess_rgb():
     assert guess_rgb(shape)
 
 
+@given(shape=array_shapes(min_dims=3, min_side=0))
+def test_guess_rgb_property(shape):
+    assert guess_rgb(shape) == (shape[-1] in (3, 4))
+
+
 def test_guess_multiscale():
     data = np.random.random((10, 15))
     assert not guess_multiscale(data)[0]
@@ -71,7 +78,9 @@ def test_guess_multiscale_strip_single_scale():
 
 def test_guess_multiscale_non_array_list():
     """Check that non-decreasing list input raises ValueError"""
-    data = [np.empty((10, 15, 6)),] * 2  # noqa: E231
+    data = [
+        np.empty((10, 15, 6)),
+    ] * 2  # noqa: E231
     with pytest.raises(ValueError):
         _, _ = guess_multiscale(data)
 
