@@ -1,7 +1,9 @@
 import numpy as np
 from skimage.transform import pyramid_gaussian
-from vispy.color import Colormap
+
+from napari._tests.utils import check_layer_world_data_extent
 from napari.layers import Image
+from napari.utils import Colormap
 
 
 def test_random_multiscale():
@@ -13,7 +15,7 @@ def test_random_multiscale():
     assert layer.data == data
     assert layer.multiscale is True
     assert layer.ndim == len(shapes[0])
-    assert layer.shape == shapes[0]
+    np.testing.assert_array_equal(layer.extent.data[1], shapes[0])
     assert layer.rgb is False
     assert layer._data_view.ndim == 2
 
@@ -27,7 +29,7 @@ def test_infer_multiscale():
     assert layer.data == data
     assert layer.multiscale is True
     assert layer.ndim == len(shapes[0])
-    assert layer.shape == shapes[0]
+    np.testing.assert_array_equal(layer.extent.data[1], shapes[0])
     assert layer.rgb is False
     assert layer._data_view.ndim == 2
 
@@ -41,7 +43,7 @@ def test_infer_tuple_multiscale():
     assert layer.data == data
     assert layer.multiscale is True
     assert layer.ndim == len(shapes[0])
-    assert layer.shape == shapes[0]
+    np.testing.assert_array_equal(layer.extent.data[1], shapes[0])
     assert layer.rgb is False
     assert layer._data_view.ndim == 2
 
@@ -55,7 +57,7 @@ def test_blocking_multiscale():
     assert np.all(layer.data == data)
     assert layer.multiscale is False
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.ndim == 2
 
@@ -70,7 +72,7 @@ def test_multiscale_tuple():
     assert layer.data == data
     assert layer.multiscale is True
     assert layer.ndim == len(shape)
-    assert layer.shape == shape
+    np.testing.assert_array_equal(layer.extent.data[1], shape)
     assert layer.rgb is False
     assert layer._data_view.ndim == 2
 
@@ -83,7 +85,7 @@ def test_3D_multiscale():
     layer = Image(data, multiscale=True)
     assert layer.data == data
     assert layer.ndim == len(shapes[0])
-    assert layer.shape == shapes[0]
+    np.testing.assert_array_equal(layer.extent.data[1], shapes[0])
     assert layer.rgb is False
     assert layer._data_view.ndim == 2
 
@@ -96,7 +98,7 @@ def test_non_uniform_3D_multiscale():
     layer = Image(data, multiscale=True)
     assert layer.data == data
     assert layer.ndim == len(shapes[0])
-    assert layer.shape == shapes[0]
+    np.testing.assert_array_equal(layer.extent.data[1], shapes[0])
     assert layer.rgb is False
     assert layer._data_view.ndim == 2
 
@@ -109,7 +111,7 @@ def test_rgb_multiscale():
     layer = Image(data, multiscale=True)
     assert layer.data == data
     assert layer.ndim == len(shapes[0]) - 1
-    assert layer.shape == shapes[0][:-1]
+    np.testing.assert_array_equal(layer.extent.data[1], shapes[0][:-1])
     assert layer.rgb is True
     assert layer._data_view.ndim == 3
 
@@ -122,7 +124,7 @@ def test_3D_rgb_multiscale():
     layer = Image(data, multiscale=True)
     assert layer.data == data
     assert layer.ndim == len(shapes[0]) - 1
-    assert layer.shape == shapes[0][:-1]
+    np.testing.assert_array_equal(layer.extent.data[1], shapes[0][:-1])
     assert layer.rgb is True
     assert layer._data_view.ndim == 3
 
@@ -135,7 +137,7 @@ def test_non_rgb_image():
     layer = Image(data, multiscale=True, rgb=False)
     assert layer.data == data
     assert layer.ndim == len(shapes[0])
-    assert layer.shape == shapes[0]
+    np.testing.assert_array_equal(layer.extent.data[1], shapes[0])
     assert layer.rgb is False
 
 
@@ -229,36 +231,36 @@ def test_colormaps():
     np.random.seed(0)
     data = [np.random.random(s) for s in shapes]
     layer = Image(data, multiscale=True)
-    assert layer.colormap[0] == 'gray'
-    assert type(layer.colormap[1]) == Colormap
+    assert layer.colormap.name == 'gray'
+    assert isinstance(layer.colormap, Colormap)
 
     layer.colormap = 'magma'
-    assert layer.colormap[0] == 'magma'
-    assert type(layer.colormap[1]) == Colormap
+    assert layer.colormap.name == 'magma'
+    assert isinstance(layer.colormap, Colormap)
 
     cmap = Colormap([[0.0, 0.0, 0.0, 0.0], [0.3, 0.7, 0.2, 1.0]])
     layer.colormap = 'custom', cmap
-    assert layer.colormap[0] == 'custom'
-    assert layer.colormap[1] == cmap
+    assert layer.colormap.name == 'custom'
+    assert layer.colormap == cmap
 
     cmap = Colormap([[0.0, 0.0, 0.0, 0.0], [0.7, 0.2, 0.6, 1.0]])
     layer.colormap = {'new': cmap}
-    assert layer.colormap[0] == 'new'
-    assert layer.colormap[1] == cmap
+    assert layer.colormap.name == 'new'
+    assert layer.colormap == cmap
 
     layer = Image(data, multiscale=True, colormap='magma')
-    assert layer.colormap[0] == 'magma'
-    assert type(layer.colormap[1]) == Colormap
+    assert layer.colormap.name == 'magma'
+    assert isinstance(layer.colormap, Colormap)
 
     cmap = Colormap([[0.0, 0.0, 0.0, 0.0], [0.3, 0.7, 0.2, 1.0]])
     layer = Image(data, multiscale=True, colormap=('custom', cmap))
-    assert layer.colormap[0] == 'custom'
-    assert layer.colormap[1] == cmap
+    assert layer.colormap.name == 'custom'
+    assert layer.colormap == cmap
 
     cmap = Colormap([[0.0, 0.0, 0.0, 0.0], [0.7, 0.2, 0.6, 1.0]])
     layer = Image(data, multiscale=True, colormap={'new': cmap})
-    assert layer.colormap[0] == 'new'
-    assert layer.colormap[1] == cmap
+    assert layer.colormap.name == 'new'
+    assert layer.colormap == cmap
 
 
 def test_contrast_limits():
@@ -270,7 +272,6 @@ def test_contrast_limits():
     assert layer.contrast_limits[0] >= 0
     assert layer.contrast_limits[1] <= 1
     assert layer.contrast_limits[0] < layer.contrast_limits[1]
-    assert layer.contrast_limits == layer._contrast_limits_range
 
     # Change contrast_limits property
     contrast_limits = [0, 2]
@@ -321,8 +322,7 @@ def test_value():
     np.random.seed(0)
     data = [np.random.random(s) for s in shapes]
     layer = Image(data, multiscale=True)
-    value = layer.get_value()
-    assert layer.coordinates == (0, 0)
+    value = layer.get_value((0,) * 2)
     assert layer.data_level == 2
     np.testing.assert_allclose(value, (2, data[2][0, 0]))
 
@@ -333,7 +333,7 @@ def test_corner_value():
     np.random.seed(0)
     data = [np.random.random(s) for s in shapes]
     layer = Image(data, multiscale=True)
-    value = layer.get_value()
+    value = layer.get_value((0,) * 2)
     target_position = (39, 19)
     target_level = 0
     layer.data_level = target_level
@@ -341,15 +341,13 @@ def test_corner_value():
     layer.refresh()
 
     # Test position at corner of image
-    layer.position = target_position
-    value = layer.get_value()
+    value = layer.get_value(target_position)
     np.testing.assert_allclose(
         value, (target_level, data[target_level][target_position])
     )
 
     # Test position at outside image
-    layer.position = (40, 20)
-    value = layer.get_value()
+    value = layer.get_value((40, 20))
     assert value[1] is None
 
 
@@ -359,7 +357,7 @@ def test_message():
     np.random.seed(0)
     data = [np.random.random(s) for s in shapes]
     layer = Image(data, multiscale=True)
-    msg = layer.get_message()
+    msg = layer.get_status((0,) * 2)
     assert type(msg) == str
 
 
@@ -381,3 +379,24 @@ def test_not_create_random_multiscale():
     layer = Image(data)
     assert np.all(layer.data == data)
     assert layer.multiscale is False
+
+
+def test_world_data_extent():
+    """Test extent after applying transforms."""
+    np.random.seed(0)
+    shapes = [(6, 40, 80), (3, 20, 40), (1, 10, 20)]
+    data = [np.random.random(s) for s in shapes]
+    layer = Image(data)
+    extent = np.array(((0,) * 3, shapes[0]))
+    check_layer_world_data_extent(layer, extent, (3, 1, 1), (10, 20, 5), True)
+
+
+def test_5D_multiscale():
+    """Test 5D multiscale data."""
+    shapes = [(1, 2, 5, 20, 20), (1, 2, 5, 10, 10), (1, 2, 5, 5, 5)]
+    np.random.seed(0)
+    data = [np.random.random(s) for s in shapes]
+    layer = Image(data, multiscale=True)
+    assert layer.data == data
+    assert layer.multiscale is True
+    assert layer.ndim == len(shapes[0])
