@@ -1,7 +1,9 @@
-from enum import auto
 import sys
+from collections import OrderedDict
+from enum import auto
 
 from ...utils.misc import StringEnum
+from ...utils.translations import trans
 
 
 class Mode(StringEnum):
@@ -42,23 +44,19 @@ class LabelColorMode(StringEnum):
     AUTO (default) allows color to be set via a hash function with a seed.
 
     DIRECT allows color of each label to be set directly by a color dictionary.
+
+    SELECTED allows only selected labels to be visible
     """
 
     AUTO = auto()
     DIRECT = auto()
 
 
-class LabelBrushShape(StringEnum):
-    """
-    LabelBrushShape: Labelling brush shape.
-
-    CIRCLE (default) uses circle paintbrush (case insensitive).
-
-    SQUARE uses square paintbrush (case insensitive).
-    """
-
-    CIRCLE = auto()
-    SQUARE = auto()
-
-
 BACKSPACE = 'delete' if sys.platform == 'darwin' else 'backspace'
+
+LABEL_COLOR_MODE_TRANSLATIONS = OrderedDict(
+    [
+        (LabelColorMode.AUTO, trans._("auto")),
+        (LabelColorMode.DIRECT, trans._("direct")),
+    ]
+)
