@@ -1,34 +1,11 @@
+from typing import Tuple
+
 import numpy as np
-from ..utils.layer_utils import segment_normal
 from vispy.geometry import PolygonData
 from vispy.visuals.tube import _frenet_frames
 
-
-def inside_triangles(triangles):
-    """Checks which triangles contain the origin
-
-    Parameters
-    ----------
-    triangles : (N, 3, 2) array
-        Array of N triangles that should be checked
-
-    Returns
-    -------
-    inside : (N,) array of bool
-        Array with `True` values for trinagles containing the origin
-    """
-
-    AB = triangles[:, 1, :] - triangles[:, 0, :]
-    AC = triangles[:, 2, :] - triangles[:, 0, :]
-    BC = triangles[:, 2, :] - triangles[:, 1, :]
-
-    s_AB = -AB[:, 0] * triangles[:, 0, 1] + AB[:, 1] * triangles[:, 0, 0] >= 0
-    s_AC = -AC[:, 0] * triangles[:, 0, 1] + AC[:, 1] * triangles[:, 0, 0] >= 0
-    s_BC = -BC[:, 0] * triangles[:, 1, 1] + BC[:, 1] * triangles[:, 1, 0] >= 0
-
-    inside = np.all(np.array([s_AB != s_AC, s_AB == s_BC]), axis=0)
-
-    return inside
+from ...utils.translations import trans
+from ..utils.layer_utils import segment_normal
 
 
 def inside_boxes(boxes):
@@ -396,8 +373,10 @@ def rectangle_to_box(data):
     """
     if not data.shape[0] == 4:
         raise ValueError(
-            """Data shape does not match expected `[4, D]`
-                         shape specifying corners for the rectangle"""
+            trans._(
+                "Data shape does not match expected `[4, D]` shape specifying corners for the rectangle",
+                deferred=True,
+            )
         )
     box = np.array(
         [
@@ -427,7 +406,7 @@ def find_corners(data):
     Returns
     -------
     corners : np.ndarray
-        4x2 array of corners of the boudning box
+        4x2 array of corners of the bounding box
     """
     min_val = data.min(axis=0)
     max_val = data.max(axis=0)
@@ -452,7 +431,7 @@ def center_radii_to_corners(center, radii):
     Returns
     -------
     corners : np.ndarray
-        4x2 array of corners of the boudning box
+        4x2 array of corners of the bounding box
     """
     data = np.array([center + radii, center - radii])
     corners = find_corners(data)
@@ -489,8 +468,10 @@ def triangulate_ellipse(corners, num_segments=100):
     """
     if not corners.shape[0] == 4:
         raise ValueError(
-            """Data shape does not match expected `[4, D]`
-                         shape specifying corners for the ellipse"""
+            trans._(
+                "Data shape does not match expected `[4, D]` shape specifying corners for the ellipse",
+                deferred=True,
+            )
         )
 
     center = corners.mean(axis=0)
@@ -547,7 +528,7 @@ def triangulate_face(data):
     Returns
     -------
     vertices : np.ndarray
-        Mx2 array vertices of the trinagles.
+        Mx2 array vertices of the triangles.
     triangles : np.ndarray
         Px3 array of the indices of the vertices that will form the
         triangles of the triangulation
@@ -576,7 +557,7 @@ def triangulate_edge(path, closed=False):
     Returns
     -------
     centers : np.ndarray
-        Mx2 or Mx3 array central coordinates of path trinagles.
+        Mx2 or Mx3 array central coordinates of path triangles.
     offsets : np.ndarray
         Mx2 or Mx3 array of the offsets to the central coordinates that need to
         be scaled by the line width and then added to the centers to
@@ -635,7 +616,7 @@ def generate_2D_edge_meshes(path, closed=False, limit=3, bevel=False):
     Returns
     -------
     centers : np.ndarray
-        Mx2 or Mx3 array central coordinates of path trinagles.
+        Mx2 or Mx3 array central coordinates of path triangles.
     offsets : np.ndarray
         Mx2 or Mx3 array of the offsets to the central coordinates that need to
         be scaled by the line width and then added to the centers to
@@ -781,7 +762,7 @@ def generate_tube_meshes(path, closed=False, tube_points=10):
     """Generates list of mesh vertices and triangles from a path
 
     Adapted from vispy.visuals.TubeVisual
-    https://github.com/vispy/vispy/blob/master/vispy/visuals/tube.py
+    https://github.com/vispy/vispy/blob/main/vispy/visuals/tube.py
 
     Parameters
     ----------
@@ -820,7 +801,7 @@ def generate_tube_meshes(path, closed=False, tube_points=10):
         binormal = binormals[i]
 
         # Add a vertex for each point on the circle
-        v = np.arange(tube_points, dtype=np.float) / tube_points * 2 * np.pi
+        v = np.arange(tube_points, dtype=float) / tube_points * 2 * np.pi
         cx = -1.0 * np.cos(v)
         cy = np.sin(v)
         grid[i] = pos
@@ -978,6 +959,58 @@ def points_in_poly(points, vertices):
     return inside
 
 
+def extract_shape_type(data, shape_type=None):
+    """Separates shape_type from data if present, and returns both.
+
+    Parameters
+    ----------
+    data : Array | Tuple(Array,str) | List[Array | Tuple(Array, str)] | Tuple(List[Array], str)
+        list or array of vertices belonging to each shape, optionally containing shape type strings
+    shape_type : str | None
+        metadata shape type string, or None if none was passed
+
+    Returns
+    -------
+    data : Array | List[Array]
+        list or array of vertices belonging to each shape
+    shape_type : List[str] | None
+        type of each shape in data, or None if none was passed
+    """
+    # Tuple for one shape or list of shapes with shape_type
+    if isinstance(data, Tuple):
+        shape_type = data[1]
+        data = data[0]
+    # List of (vertices, shape_type) tuples
+    elif len(data) != 0 and all(isinstance(datum, Tuple) for datum in data):
+        shape_type = [datum[1] for datum in data]
+        data = [datum[0] for datum in data]
+    return data, shape_type
+
+
+def get_default_shape_type(current_type):
+    """If all shapes in current_type are of identical shape type,
+       return this shape type, else "polygon" as lowest common
+       denominator type.
+
+    Parameters
+    ----------
+    current_type : list of str
+        list of current shape types
+
+    Returns
+    -------
+    default_type : str
+        default shape type
+    """
+    default = "polygon"
+    if not current_type:
+        return default
+    first_type = current_type[0]
+    if all(shape_type == first_type for shape_type in current_type):
+        return first_type
+    return default
+
+
 def get_shape_ndim(data):
     """Checks whether data is a list of the same type of shape, one shape, or
     a list of different shapes and returns the dimensionality of the shape/s.
@@ -994,7 +1027,7 @@ def get_shape_ndim(data):
         Dimensionality of the shape/s in data
     """
     # list of all the same shapes
-    if np.array(data).ndim == 3:
+    if np.array(data, dtype=object).ndim == 3:
         ndim = np.array(data).shape[2]
     # just one shape
     elif np.array(data[0]).ndim == 1:
@@ -1003,3 +1036,79 @@ def get_shape_ndim(data):
     else:
         ndim = np.array(data[0]).shape[1]
     return ndim
+
+
+def number_of_shapes(data):
+    """Determine number of shapes in the data.
+
+    Parameters
+    ----------
+    data : list or np.ndarray
+        Can either be no shapes, if empty, a
+        single shape or a list of shapes.
+
+    Returns
+    -------
+    n_shapes : int
+        Number of new shapes
+    """
+    if len(data) == 0:
+        # If no new shapes
+        n_shapes = 0
+    elif np.array(data[0]).ndim == 1:
+        # If a single array for a shape
+        n_shapes = 1
+    else:
+        n_shapes = len(data)
+
+    return n_shapes
+
+
+def validate_num_vertices(
+    data, shape_type, min_vertices=None, valid_vertices=None
+):
+    """Raises error if a shape in data has invalid number of vertices.
+
+    Checks whether all shapes in data have a valid number of vertices
+    for the given shape type and vertex information. Rectangles and
+    ellipses can have either 2 or 4 vertices per shape,
+    lines can have only 2, while paths and polygons have a minimum
+    number of vertices, but no maximum.
+
+    One of valid_vertices or min_vertices must be passed to the
+    function.
+
+    Parameters
+    ----------
+    data : Array | Tuple(Array,str) | List[Array | Tuple(Array, str)] | Tuple(List[Array], str)
+        List of shape data, where each element is either an (N, D) array of the
+        N vertices of a shape in D dimensions or a tuple containing an array of
+        the N vertices and the shape_type string. Can be an 3-dimensional array
+        if each shape has the same number of vertices.
+    shape_type : str
+        Type of shape being validated (for detailed error message)
+    min_vertices : int or None
+        Minimum number of vertices for the shape type, by default None
+    valid_vertices : Tuple(int) or None
+        Valid number of vertices for the shape type in data, by default None
+
+    Raises
+    ------
+    ValueError
+        Raised if a shape is found with invalid number of vertices
+    """
+    n_shapes = number_of_shapes(data)
+    # single array of vertices
+    if n_shapes == 1 and len(np.array(data).shape) == 2:
+        # wrap in extra dimension so we can iterate through shape not vertices
+        data = [data]
+    for shape in data:
+        if (valid_vertices and len(shape) not in valid_vertices) or (
+            min_vertices and len(shape) < min_vertices
+        ):
+            raise ValueError(
+                trans._(
+                    f"{shape_type} {shape} has invalid number of vertices: {len(shape)}.",
+                    deferred=True,
+                )
+            )
