@@ -221,10 +221,14 @@ class ColorManager(EventedModel):
         # if the provided color is a string, first check if it is a key in the properties.
         # otherwise, assume it is the name of a color
         if is_color_mapped(color, properties):
+            # note that we set ColorProperties.current_value by indexing rather than
+            # np.squeeze since the current_property values have shape (1,) and
+            # np.squeeze would return an array with shape ().
+            # see https://github.com/napari/napari/pull/3110#discussion_r680680779
             self.color_properties = ColorProperties(
                 name=color,
                 values=properties[color],
-                current_value=np.squeeze(current_properties[color]),
+                current_value=current_properties[color][0],
             )
             if guess_continuous(properties[color]):
                 self.color_mode = ColorMode.COLORMAP
@@ -405,9 +409,14 @@ class ColorManager(EventedModel):
             current_property_name = self.color_properties.name
             current_property_values = self.color_properties.values
             if current_property_name in current_properties:
-                new_current_value = np.squeeze(
-                    current_properties[current_property_name]
-                )
+                # note that we set ColorProperties.current_value by indexing rather than
+                # np.squeeze since the current_property values have shape (1,) and
+                # np.squeeze would return an array with shape ().
+                # see https://github.com/napari/napari/pull/3110#discussion_r680680779
+                new_current_value = current_properties[current_property_name][
+                    0
+                ]
+
                 if new_current_value != self.color_properties.current_value:
                     self.color_properties = ColorProperties(
                         name=current_property_name,
