@@ -1,16 +1,49 @@
-from typing import List, Dict
+from __future__ import annotations
+
 import itertools
+from typing import TYPE_CHECKING, List
 
 import numpy as np
+
 from ...layers import Image
-from ...utils.colormaps import colormaps
 from ...layers.image._image_utils import guess_multiscale
+from ...utils.colormaps import CYMRGB, MAGENTA_GREEN, Colormap
 from ...utils.misc import ensure_iterable, ensure_sequence_of_iterables
-from ...types import FullLayerData
+from ...utils.translations import trans
+
+if TYPE_CHECKING:
+    from ...types import FullLayerData
+
+
+def slice_from_axis(array, *, axis, element):
+    """Take a single index slice from array using slicing.
+
+    Equivalent to :func:`np.take`, but using slicing, which ensures that the
+    output is a view of the original array.
+
+    Parameters
+    ----------
+    array : NumPy or other array
+        Input array to be sliced.
+    axis : int
+        The axis along which to slice.
+    element : int
+        The element along that axis to grab.
+
+    Returns
+    -------
+    sliced : NumPy or other array
+        The sliced output array, which has one less dimension than the input.
+    """
+    slices = [slice(None) for i in range(array.ndim)]
+    slices[axis] = element
+    return array[tuple(slices)]
 
 
 def split_channels(
-    data: np.ndarray, channel_axis: int, **kwargs,
+    data: np.ndarray,
+    channel_axis: int,
+    **kwargs,
 ) -> List[FullLayerData]:
     """Split the data array into separate arrays along an axis.
 
@@ -31,7 +64,7 @@ def split_channels(
     data : array or list of array
     channel_axis : int
         Axis to split the image along.
-    kwargs: dict
+    **kwargs : dict
         Keyword arguments will override the default image meta keys
         returned in each layer data tuple.
 
@@ -51,7 +84,15 @@ def split_channels(
     kwargs['blending'] = kwargs.get('blending') or 'additive'
     kwargs.setdefault('colormap', None)
     # these arguments are *already* iterables in the single-channel case.
-    iterable_kwargs = {'scale', 'translate', 'contrast_limits', 'metadata'}
+    iterable_kwargs = {
+        'scale',
+        'translate',
+        'affine',
+        'contrast_limits',
+        'metadata',
+        'experimental_slicing_plane',
+        'experimental_clipping_planes',
+    }
 
     # turn the kwargs dict into a mapping of {key: iterator}
     # so that we can use {k: next(v) for k, v in kwargs.items()} below
@@ -60,15 +101,21 @@ def split_channels(
             if n_channels == 1:
                 kwargs[key] = iter(['gray'])
             elif n_channels == 2:
-                kwargs[key] = iter(colormaps.MAGENTA_GREEN)
+                kwargs[key] = iter(MAGENTA_GREEN)
             else:
-                kwargs[key] = itertools.cycle(colormaps.CYMRGB)
+                kwargs[key] = itertools.cycle(CYMRGB)
 
         # make sure that iterable_kwargs are a *sequence* of iterables
         # for the multichannel case.  For example: if scale == (1, 2) &
         # n_channels = 3, then scale should == [(1, 2), (1, 2), (1, 2)]
-        elif key in iterable_kwargs:
-            kwargs[key] = iter(ensure_sequence_of_iterables(val, n_channels))
+        elif key in iterable_kwargs or (
+            key == 'colormap' and isinstance(val, Colormap)
+        ):
+            kwargs[key] = iter(
+                ensure_sequence_of_iterables(
+                    val, n_channels, repeat_empty=True
+                )
+            )
         else:
             kwargs[key] = iter(ensure_iterable(val))
 
@@ -76,21 +123,26 @@ def split_channels(
     for i in range(n_channels):
         if multiscale:
             image = [
-                np.take(data[j], i, axis=channel_axis)
+                slice_from_axis(data[j], axis=channel_axis, element=i)
                 for j in range(len(data))
             ]
         else:
-            image = np.take(data, i, axis=channel_axis)
+            image = slice_from_axis(data, axis=channel_axis, element=i)
         i_kwargs = {}
         for key, val in kwargs.items():
             try:
                 i_kwargs[key] = next(val)
             except StopIteration:
                 raise IndexError(
-                    "Error adding multichannel image with data shape "
-                    f"{data.shape!r}.\nRequested channel_axis "
-                    f"({channel_axis}) had length {n_channels}, but "
-                    f"the '{key}' argument only provided {i} values. "
+                    trans._(
+                        "Error adding multichannel image with data shape {data_shape!r}.\nRequested channel_axis ({channel_axis}) had length {n_channels}, but the '{key}' argument only provided {i} values. ",
+                        deferred=True,
+                        data_shape=data.shape,
+                        channel_axis=channel_axis,
+                        n_channels=n_channels,
+                        key=key,
+                        i=i,
+                    )
                 )
 
         layerdata = (image, i_kwargs, 'image')
@@ -99,7 +151,7 @@ def split_channels(
     return layerdata_list
 
 
-def stack_to_images(stack: Image, axis: int, **kwargs: Dict,) -> List[Image]:
+def stack_to_images(stack: Image, axis: int, **kwargs) -> List[Image]:
     """Splits a single Image layer into a list layers along axis.
 
     Some image layer properties will be changed unless specified as an item in
@@ -131,17 +183,23 @@ def stack_to_images(stack: Image, axis: int, **kwargs: Dict,) -> List[Image]:
         del meta[key]
 
     name = stack.name
-    num_dim = stack.dims.ndim
+    num_dim = 3 if stack.rgb else stack.ndim
 
     if num_dim < 3:
         raise ValueError(
-            "The image needs more than 2 dimensions for splitting",
+            trans._(
+                "The image needs more than 2 dimensions for splitting",
+                deferred=True,
+            )
         )
 
     if axis >= num_dim:
         raise ValueError(
-            "Can't split along axis {}. The image has {} dimensions".format(
-                axis, num_dim
+            trans._(
+                "Can't split along axis {axis}. The image has {num_dim} dimensions",
+                deferred=True,
+                axis=axis,
+                num_dim=num_dim,
             )
         )
 
@@ -149,7 +207,7 @@ def stack_to_images(stack: Image, axis: int, **kwargs: Dict,) -> List[Image]:
         kwargs['colormap'] = itertools.cycle(kwargs['colormap'])
 
     if meta['rgb']:
-        if axis == (num_dim - 1) or axis == -1:
+        if axis in [num_dim - 1, -1]:
             kwargs['rgb'] = False  # split channels as grayscale
         else:
             kwargs['rgb'] = True  # split some other axis, remain rgb
@@ -160,8 +218,12 @@ def stack_to_images(stack: Image, axis: int, **kwargs: Dict,) -> List[Image]:
         meta['scale'].pop(axis)
         meta['translate'].pop(axis)
 
+    meta['rotate'] = None
+    meta['shear'] = None
+    meta['affine'] = None
+
     meta.update(kwargs)
-    imagelist = list()
+    imagelist = []
     layerdata_list = split_channels(data, axis, **meta)
     for i, tup in enumerate(layerdata_list):
         idata, imeta, _ = tup
@@ -173,9 +235,15 @@ def stack_to_images(stack: Image, axis: int, **kwargs: Dict,) -> List[Image]:
     return imagelist
 
 
-def images_to_stack(
-    images: List[Image], axis: int = 0, **kwargs: Dict,
-) -> Image:
+def split_rgb(stack: Image, with_alpha=False) -> List[Image]:
+    """Variant of stack_to_images that splits an RGB with predefined cmap."""
+    if not stack.rgb:
+        raise ValueError('Image must be RGB to use split_rgb')
+    images = stack_to_images(stack, -1, colormap=('red', 'green', 'blue'))
+    return images if with_alpha else images[:3]
+
+
+def images_to_stack(images: List[Image], axis: int = 0, **kwargs) -> Image:
     """Combines a list of Image layers into one layer stacked along axis
 
     The new image layer will get the meta properties of the first
@@ -187,18 +255,18 @@ def images_to_stack(
         List of Image Layers
     axis : int
         Index to to insert the new axis
-    kwargs : dict
+    **kwargs : dict
         Dictionary of parameters values to override parameters
         from the first image in images list.
 
-    Returns.
+    Returns
     -------
     stack : napari.layers.Image
         Combined image stack
     """
 
-    if len(images) == 0:
-        raise IndexError("images list is empty")
+    if not images:
+        raise IndexError(trans._("images list is empty", deferred=True))
 
     data, meta, _ = images[0].as_layer_data_tuple()
 
@@ -207,6 +275,11 @@ def images_to_stack(
 
     meta.update(kwargs)
     new_data = np.stack([image.data for image in images], axis=axis)
-    stack = Image(new_data, **meta)
+    return Image(new_data, **meta)
 
-    return stack
+
+def merge_rgb(images: List[Image]) -> List[Image]:
+    """Variant of images_to_stack that makes an RGB from 3 images."""
+    if not (len(images) == 3 and all(isinstance(x, Image) for x in images)):
+        raise ValueError("merge_rgb requires 3 images layers")
+    return images_to_stack(images, axis=-1, rgb=True)
