@@ -1,32 +1,52 @@
+import os
+import pathlib
 import warnings
+from collections import namedtuple
 from logging import getLogger
-from typing import List, Optional, Sequence, Union
+from typing import Any, List, Optional, Sequence, Tuple, Union
 
-from napari_plugin_engine import (
-    HookImplementation,
-    PluginCallError,
-    PluginManager,
-)
+from napari_plugin_engine import HookImplementation, PluginCallError
 
 from ..layers import Layer
 from ..types import LayerData
 from ..utils.misc import abspath_or_url
-from . import plugin_manager as napari_plugin_manager
+from ..utils.translations import trans
+from . import plugin_manager
 
 logger = getLogger(__name__)
 
 
+def _read_with_npe2(path, plugin):
+    """Try to return data for `path`, from reader plugins using a manifest."""
+    try:
+        from npe2 import execute_command, plugin_manager
+    except ImportError:
+        return
+
+    for rdr in plugin_manager.iter_compatible_readers(path):
+        read_func = execute_command(rdr.command, kwargs={'path': path})
+        if read_func:
+            try:
+                layer_data = read_func(path)  # try to read data
+                if layer_data:
+                    # hookimpl just mocks ``.plugin_name` attribute access
+                    # until we drop support for the old hookimpl stuff.
+                    hookimpl = namedtuple('hookimpl', ('plugin_name'))
+                    return layer_data, hookimpl(rdr.command.split(".")[0])
+            except Exception:
+                continue
+
+
 def read_data_with_plugins(
     path: Union[str, Sequence[str]],
     plugin: Optional[str] = None,
-    plugin_manager: PluginManager = napari_plugin_manager,
-) -> List[LayerData]:
+) -> Tuple[Optional[List[LayerData]], Optional[HookImplementation]]:
     """Iterate reader hooks and return first non-None LayerData or None.
 
     This function returns as soon as the path has been read successfully,
-    while catching any plugin exceptions, storing them for later retrievial,
-    providing useful error messages, and relooping until either layer data is
-    returned, or no valid readers are found.
+    while catching any plugin exceptions, storing them for later retrieval,
+    providing useful error messages, and re-looping until either a read
+    operation was successful, or no valid readers were found.
 
     Exceptions will be caught and stored as PluginErrors
     (in plugins.exceptions.PLUGIN_ERRORS)
@@ -39,41 +59,67 @@ def read_data_with_plugins(
         Name of a plugin to use.  If provided, will force ``path`` to be read
         with the specified ``plugin``.  If the requested plugin cannot read
         ``path``, a PluginCallError will be raised.
-    plugin_manager : plugins.PluginManager, optional
-        Instance of a napari PluginManager.  by default the main napari
-        plugin_manager will be used.
 
     Returns
     -------
     LayerData : list of tuples, or None
         LayerData that can be passed to :func:`Viewer._add_layer_from_data()
-        <napari.components.add_layers_mixin.AddLayersMixin._add_layer_from_data>`.
+        <napari.components.viewer_model.ViewerModel._add_layer_from_data>`.
         ``LayerData`` is a list tuples, where each tuple is one of
         ``(data,)``, ``(data, meta)``, or ``(data, meta, layer_type)`` .
 
-        If no reader plugins are (or they all error), returns ``None``
+        If no reader plugins were found (or they all failed), returns ``None``
 
     Raises
     ------
     PluginCallError
         If ``plugin`` is specified but raises an Exception while reading.
     """
-    hook_caller = plugin_manager.hook.napari_get_reader
 
+    _ld = _read_with_npe2(path, plugin)
+    if _ld is not None:
+        _ld, hookimpl = _ld
+        return [] if _is_null_layer_sentinel(_ld) else _ld or None, hookimpl
+
+    hook_caller = plugin_manager.hook.napari_get_reader
+    path = abspath_or_url(path)
+    if not plugin and isinstance(path, (str, pathlib.Path)):
+        extension = os.path.splitext(path)[-1]
+        plugin = plugin_manager.get_reader_for_extension(extension)
+
+    hookimpl: Optional[HookImplementation] = None
     if plugin:
         if plugin not in plugin_manager.plugins:
             names = {i.plugin_name for i in hook_caller.get_hookimpls()}
             raise ValueError(
-                f"There is no registered plugin named '{plugin}'.\n"
-                f"Names of plugins offering readers are: {names}"
+                trans._(
+                    "There is no registered plugin named '{plugin}'.\nNames of plugins offering readers are: {names}",
+                    deferred=True,
+                    plugin=plugin,
+                    names=names,
+                )
             )
         reader = hook_caller._call_plugin(plugin, path=path)
         if not callable(reader):
-            raise ValueError(f'Plugin {plugin!r} does not support file {path}')
-        return reader(path) or []
+            raise ValueError(
+                trans._(
+                    'Plugin {plugin!r} does not support file {path}',
+                    deferred=True,
+                    plugin=plugin,
+                    path=path,
+                )
+            )
+
+        hookimpl = hook_caller.get_plugin_implementation(plugin)
+        layer_data = reader(path)
+        # if the reader returns a "null layer" sentinel indicating an empty
+        # file, return an empty list, otherwise return the result or None
+        if _is_null_layer_sentinel(layer_data):
+            return [], hookimpl
+
+        return layer_data or None, hookimpl
 
     errors: List[PluginCallError] = []
-    path = abspath_or_url(path)
     skip_impls: List[HookImplementation] = []
     layer_data = None
     while True:
@@ -87,6 +133,7 @@ def read_data_with_plugins(
         try:
             layer_data = reader(path)  # try to read data
             if layer_data:
+                hookimpl = result.implementation
                 break
         except Exception as exc:
             # collect the error and log it, but don't raise it.
@@ -101,24 +148,39 @@ def read_data_with_plugins(
         # we just want to provide some useful feedback, which includes
         # whether or not paths were passed to plugins as a list.
         if isinstance(path, (tuple, list)):
-            path_repr = f"[{path[0]}, ...] as stack"
+            message = trans._(
+                'No plugin found capable of reading [{repr_path}, ...] as stack.',
+                deferred=True,
+                repr_path=path[0],
+            )
         else:
-            path_repr = repr(path)
-        msg = f'No plugin found capable of reading {path_repr}.'
-        logger.warn(msg)
+            message = trans._(
+                'No plugin found capable of reading {repr_path}.',
+                deferred=True,
+                repr_path=repr(path),
+            )
+
+        # TODO: change to a warning notification in a later PR
+        raise ValueError(message)
 
     if errors:
-        names = set([repr(e.plugin_name) for e in errors])
+        names = {repr(e.plugin_name) for e in errors}
         err_msg = f"({len(errors)}) error{'s' if len(errors) > 1 else ''} "
         err_msg += f"occurred in plugins: {', '.join(names)}. "
         err_msg += 'See full error logs in "Plugins → Plugin Errors..."'
         logger.error(err_msg)
 
-    return layer_data or []
+    # if the reader returns a "null layer" sentinel indicating an empty file,
+    # return an empty list, otherwise return the result or None
+    _data = [] if _is_null_layer_sentinel(layer_data) else layer_data or None
+    return _data, hookimpl
 
 
 def save_layers(
-    path: str, layers: List[Layer], *, plugin: Optional[str] = None,
+    path: str,
+    layers: List[Layer],
+    *,
+    plugin: Optional[str] = None,
 ) -> List[str]:
     """Write list of layers or individual layer to a path using writer plugins.
 
@@ -184,19 +246,45 @@ def save_layers(
         # path/layers combination
         # we just want to provide some useful feedback
         warnings.warn(
-            'No data written! There may be no plugins '
-            f'capable of writing these {len(layers)} layers to {path}.'
+            trans._(
+                'No data written! There may be no plugins capable of writing these {length} layers to {path}.',
+                deferred=True,
+                length=len(layers),
+                path=path,
+            )
         )
 
     return written
 
 
+def _is_null_layer_sentinel(layer_data: Union[LayerData, Any]) -> bool:
+    """Checks if the layer data returned from a reader function indicates an
+    empty file. The sentinel value used for this is ``[(None,)]``.
+
+    Parameters
+    ----------
+    layer_data : LayerData
+        The layer data returned from a reader function to check
+
+    Returns
+    -------
+    bool
+        True, if the layer_data indicates an empty file, False otherwise
+    """
+    return (
+        isinstance(layer_data, list)
+        and len(layer_data) == 1
+        and isinstance(layer_data[0], tuple)
+        and len(layer_data[0]) == 1
+        and layer_data[0][0] is None
+    )
+
+
 def _write_multiple_layers_with_plugins(
     path: str,
     layers: List[Layer],
     *,
     plugin_name: Optional[str] = None,
-    plugin_manager=napari_plugin_manager,
 ) -> List[str]:
     """Write data from multiple layers data with a plugin.
 
@@ -225,9 +313,6 @@ def _write_multiple_layers_with_plugins(
         If provided, force the plugin manager to use the ``napari_get_writer``
         from the requested ``plugin_name``.  If none is available, or if it is
         incapable of handling the layers, this function will fail.
-    plugin_manager : plugins.PluginManager, optional
-        Instance of a PluginManager.  by default the main napari
-        plugin_manager will be used.
 
     Returns
     -------
@@ -237,6 +322,10 @@ def _write_multiple_layers_with_plugins(
     layer_data = [layer.as_layer_data_tuple() for layer in layers]
     layer_types = [ld[2] for ld in layer_data]
 
+    if not plugin_name and isinstance(path, (str, pathlib.Path)):
+        extension = os.path.splitext(path)[-1]
+        plugin_name = plugin_manager.get_writer_for_extension(extension)
+
     hook_caller = plugin_manager.hook.napari_get_writer
     path = abspath_or_url(path)
     if plugin_name:
@@ -245,8 +334,12 @@ def _write_multiple_layers_with_plugins(
         if plugin_name not in plugin_manager.plugins:
             names = {i.plugin_name for i in hook_caller.get_hookimpls()}
             raise ValueError(
-                f"There is no registered plugin named '{plugin_name}'.\n"
-                f"Names of plugins offering writers are: {names}"
+                trans._(
+                    "There is no registered plugin named '{plugin_name}'.\nNames of plugins offering writers are: {names}",
+                    deferred=True,
+                    plugin_name=plugin_name,
+                    names=names,
+                )
             )
         implementation = hook_caller.get_plugin_implementation(plugin_name)
         writer_function = hook_caller(
@@ -261,10 +354,19 @@ def _write_multiple_layers_with_plugins(
 
     if not callable(writer_function):
         if plugin_name:
-            msg = f'Requested plugin "{plugin_name}" is not capable'
+            msg = trans._(
+                'Requested plugin "{plugin_name}" is not capable of writing this combination of layer types: {layer_types}',
+                deferred=True,
+                plugin_name=plugin_name,
+                layer_types=layer_types,
+            )
         else:
-            msg = 'Unable to find plugin capable'
-        msg += f' of writing this combination of layer types: {layer_types}'
+            msg = trans._(
+                'Unable to find plugin capable of writing this combination of layer types: {layer_types}',
+                deferred=True,
+                layer_types=layer_types,
+            )
+
         raise ValueError(msg)
 
     try:
@@ -278,7 +380,6 @@ def _write_single_layer_with_plugins(
     layer: Layer,
     *,
     plugin_name: Optional[str] = None,
-    plugin_manager=napari_plugin_manager,
 ) -> Optional[str]:
     """Write single layer data with a plugin.
 
@@ -304,9 +405,6 @@ def _write_single_layer_with_plugins(
         Name of the plugin to write data with. If None then all plugins
         corresponding to appropriate hook specification will be looped
         through to find the first one that can write the data.
-    plugin_manager : plugins.PluginManager, optional
-        Instance of a napari PluginManager.  by default the main napari
-        plugin_manager will be used.
 
     Returns
     -------
@@ -318,12 +416,19 @@ def _write_single_layer_with_plugins(
         plugin_manager.hook, f'napari_write_{layer._type_string}'
     )
 
+    if not plugin_name and isinstance(path, (str, pathlib.Path)):
+        extension = os.path.splitext(path)[-1]
+        plugin_name = plugin_manager.get_writer_for_extension(extension)
+
     if plugin_name and (plugin_name not in plugin_manager.plugins):
         names = {i.plugin_name for i in hook_caller.get_hookimpls()}
         raise ValueError(
-            f"There is no registered plugin named '{plugin_name}'.\n"
-            "Plugins capable of writing layer._type_string layers"
-            f"are: {names}"
+            trans._(
+                "There is no registered plugin named '{plugin_name}'.\nPlugins capable of writing layer._type_string layers are: {names}",
+                deferred=True,
+                plugin_name=plugin_name,
+                names=names,
+            )
         )
 
     # Call the hook_caller
