@@ -1,4 +1,4 @@
-from functools import wraps
+from functools import partial, wraps
 from pathlib import Path
 from types import TracebackType
 from typing import (
@@ -124,23 +124,37 @@ def image_reader_to_layerdata_reader(
 
 def _register_types_with_magicgui():
     """Register napari.types objects with magicgui."""
+    import sys
+    from concurrent.futures import Future
+
+    from magicgui import register_type
+
     from . import layers
     from .utils import _magicgui as _mgui
 
-    _mgui.register_type(
-        LayerDataTuple,
-        return_callback=_mgui.add_layer_data_tuples_to_viewer,
-    )
-    _mgui.register_type(
-        List[LayerDataTuple],
-        return_callback=_mgui.add_layer_data_tuples_to_viewer,
-    )
+    for _type in (LayerDataTuple, List[LayerDataTuple]):
+        register_type(
+            _type,
+            return_callback=_mgui.add_layer_data_tuples_to_viewer,
+        )
+        if sys.version_info >= (3, 9):
+            register_type(Future[_type], return_callback=_mgui.add_future_data)
+
     for layer_name in layers.NAMES:
-        _mgui.register_type(
-            globals().get(f'{layer_name.title()}Data'),
+        data_type = globals().get(f'{layer_name.title()}Data')
+        register_type(
+            data_type,
             choices=_mgui.get_layers_data,
             return_callback=_mgui.add_layer_data_to_viewer,
         )
+        if sys.version_info >= (3, 9):
+            register_type(
+                Future[data_type],  # type: ignore
+                choices=_mgui.get_layers_data,
+                return_callback=partial(
+                    _mgui.add_future_data, _from_tuple=False
+                ),
+            )
 
 
 _register_types_with_magicgui()
